/* $Id: cartlage.cpp,v 1.1.1.1 1992/07/22 12:37:28 cvs Exp $
 *
 * Copyright (c) 1992 University of Alberta
 *
 * May be copied or modified under the terms of the GNU General Public
 * License.  See COPYING file for more information.
 */

#include <graphics.h>
#include <image.h>
#include <conio.h>
#include <fstream.h>
#include <malloc.h>
#include <mouse.h>

extern unsigned _stklen = 16384;

char* fname = "J:\\FL002.TGA";
char* datafile = "N:\\FL00.dat";
int variation = 10;
int num = 150;
int thres = 96;
int blocksize = 50;
int edgesize = 8;

ofstream *f;

long sx, sy, tr, tg, n;
int minX, maxX, minY, maxY;
int w, h;

MemoryImage *image;
MemoryImage *mask;

inline int max (int r, int g) { return (r>g ? r : g); };

int CompareIntensity (int x, int y, int c)
  {
  int r, g, b;
  image -> GetPixelTC (x, y, r, g, b);
  mask -> GetPixelGray (x, y, b);

  return (((max (r, g) >= c) && (max (r, g) < c+variation)) || (!(r+g)))
		    && (!b);
  };

int DoLine (int x, int y, int dir, int c)
  {
    int x1=x+1, x2=x, x3, x4, y1;
    int done=0, size;

    while ((CompareIntensity (x1-1, y, c)) && (x1 > 0))
      {
      x1--;
      mask -> PutPixelGray (x1, y, 255);
      };
    while ((CompareIntensity (x2+1, y, c)) && (x2<w-1))
      {
      x2++;
      mask -> PutPixelGray (x2, y, 255);
      };
    size = x2-x1+1;
    if (x1 < minX) minX = x1;
    if (x2 > maxX) maxX = x2;
    do {
      y1 = y;  (dir ? y++ : y--);
      x = x1;
      while ((!CompareIntensity (x, y, c)) && (x <= x2)) x++;
      if (x <= x2)
	{
	if (y < minY) minY=y;
	if (y > maxY) maxY=y;
	x3 = x+1;  x4=x;
	while ((CompareIntensity (x3-1, y, c)) && (x3 > 0))
	  {
	  x3--;
	  mask -> PutPixelGray (x3, y, 255);
	  };
	while ((CompareIntensity (x4+1, y, c)) && (x4 < w-1))
	  {
	  x4++;
	  mask -> PutPixelGray (x4, y, 255);
	  };
	size += x4-x3+1;
	if (x1 < x3-1)
	  for (x=x1; x<x3-1; x++)
	    if (CompareIntensity (x, y, c))
	      size += DoLine (x, y, dir, c);
	if (x4+2 <= x2)
	  for (x=x4+2; x<=x2; x++)
	    if (CompareIntensity (x, y, c))
	      size += DoLine (x, y, dir, c);
	if (x3 < x1-1)
	  for (x=x3; x<x1-1; x++)
	    if (CompareIntensity (x, y1, c))
	      size += DoLine (x, y1, !dir, c);
	if (x2+2 <= x4)
	  for (x=x2+2; x<=x4; x++)
	    if (CompareIntensity (x, y1, c))
	      size += DoLine (x, y1, !dir, c);
	x1 = x3;  x2 = x4;
	if (x1 < minX) minX = x1;
	if (x2 > maxX) maxX = x2;
	}
	else done = -1;
      if (dir && (y==h-1)) done = -1;
      if (!dir && (y==0)) done = -1;
      } while (!done);
    return size;
  };

void Fill (int x, int y, int c)
  {
  DoLine (x, y, 0, c); DoLine (x, y+1, 1, c);
  };

void FindObject (int x, int y, int c)
  {
  circle (x, y, 6);
  int j, x1, y1, c1;
    minX = maxX = x;  minY = maxY = y;
    sx=sy=tr=tg=n = 0;
    int r, g, b;
    for (j=c; j>thres; j--)
      {
      Fill (x, y, j);
      for (y1=minY-1; y1<=maxY+1; y1++)
	for (x1=minX-1; x1<=maxX+1; x1++)
	  {
	  mask -> GetPixelGray (x1, y1, c1);
	  if (c1)
	    {
	    mask -> PutPixelGray (x1, y1, 0);
	    image -> GetPixelTC (x1, y1, r, g, b);
	    image -> PutPixelTC (x1, y1, 0, 0, 0);
	    if (j==thres+1) image -> PutPixelTC (x1, y1, 0, 1, 0);
	    if (max(r, g) > c-64)
	      {
	      tr += r;  tg += g;  n++;
	      sx += x*(long)(r+g);  sy += y*(long)(r+g);
	      };
	    };
	  };
      };
    sx /= tr + tg;  sy /= tr + tg;
    image -> PutPixelTC (sx, sy, thres, thres, 0);
    *f << sx << "\t" << sy << "\t";
    *f << tr << "\t" << tg << "\t" << n << "\n";
  };

void FindMax (int x1, int y1, int x2, int y2)
  {
  int mx, my, x, y;
  int r, g, b;
  int c;

  if (x2-x1 < blocksize) x1 = x2-blocksize;
  if (y2-y1 < blocksize) y1 = y2-blocksize;
  if (x1 < 0) x1 = 0;
  if (y1 < 0) y1 = 0;
  do {
    c = 0;
    for (y=y1; y<=y2; y++)
      for (x=x1; x<=x2; x++)
	{
	image -> GetPixelTC (x, y, r, g, b);
	if (max (r, g) > c)
	  {
	  c = max (r, g);
	  mx = x;  my = y;
	  };
	};
    if (c > thres)
      {
      if ((mx > x1+edgesize) && (mx < x2-edgesize)
	  && (my > y1+edgesize) && (my < y2-edgesize))
	FindObject (mx, my, c);
      else
	if (((mx<=edgesize) && (x1==0)) || ((my<=edgesize) && (y1==0)) ||
	   ((mx>=w-edgesize-1) && (x2==w-1)) || ((my>=h-edgesize-1) && (y2==h-1)))
	  FindObject (mx, my, c);
	else FindMax ((mx>25 ? mx-25 : 0), (my>25 ? my-25 : 0),
		      (mx+25<w ? mx+25 : w-1), (my+25<h ? my+25 : h-1));
      };
    } while (c > thres);
  };

void run ()
  {
  image = new MemoryImage (8, 8, 0);
  mask = new MemoryImage (1);
  TGAFile *inimage = new TGAFile (fname, READ);

  image->LoadImage (inimage);
  delete inimage;

  image->DisplayImage (0, 0);

  int x, y, c, i;

  image -> GetImageSize (w, h);
  if (mask -> CreateImage (w, h)) return;
  mask -> ClearImage (gray, 0);

  for (x=0; x<w; x++)
    {
    image->PutPixelTC (x, 0, 1, 1, 0);
    image->PutPixelTC (x, h-1, 1, 1, 0);
    };
  for (y=0; y<h; y++)
    {
    image->PutPixelTC (0, y, 1, 1, 0),
    image->PutPixelTC (w-1, y, 1, 1, 0);
    };

  setcolor (0xFF);

  x = 0;  y = 0;

  do {
    FindMax (x, y, (x+blocksize < w ? x+blocksize : w-1),
	       (y+blocksize < h ? y+blocksize : h-1));
    x += blocksize;
    if (x>=w) x=0, y+=blocksize;
    } while (y<h);

  image -> DisplayImage (0, 0);
  delete image;
  };

void SetPalette()
  {
  int i;

  for (i=0; i<256; i++)
    setrgbpalette (i, (i&0x0F)<<2, (i&0xF0)>>2, 0);
  };

int huge detect()  {  return (2);  };

main ()
{
  int Gd = DETECT, Gm;
  int Drv;
  installuserdriver("Svga256",detect);
  initgraph(&Gd,&Gm,"P:\\BGI");
  SetPalette();
//  if (!InitMouse ()) return 1;
//  ShowMouseCursor ();
  f = new ofstream (datafile);

  run ();

  delete f;
  cout << (char) 0x07;
  getch();

  closegraph();
  return 0;
};
